{------------------------------------------------------------------------------}
{ component  : SplitterPanel                                                   }
{ version    : 1.4                                                             }
{ last update: 98/06/30                                                        }
{ written for: Borland Delphi 3.0                                              }
{ written by : Geir Wikran                                                     }
{ e-mail     : gwikran@online.no                                               }
{------------------------------------------------------------------------------}

unit SplitterPanel;

{==============================================================================}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, ExtCtrls,
  DsgnIntf, Registry, AdvancedPanel;

const
  Register_SplitterPanel = 'Installed';

type
  TSplittDirection = (spNonSplitt,spUnknown,spVertical,spHorizontal);
  TSplittBarStyle  = (spContent,spCheckered,spSolid);

  TSplitterPanel = class(TCustomAdvancedPanel)
  private
    procedure CMDesignHitTest(var Message: TCMDesignHitTest); message CM_DESIGNHITTEST;
  protected
    procedure Loaded; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
  {republished}
    property  Align;
    property  Visible;
    property  Border;
    property  BevelInner;
    property  BevelOuter;
    property  Wallpaper;
    property  OnResize;

  {splitter functionality}
  private
    FSplittState: TSplittDirection;
    FBarStyle   : TSplittBarStyle;
    FBarWidth   : Byte;
    FCursorVert : TCursor;
    FCursorHorz : TCursor;
    FSplittPanes: TList;
    FSortedPanes: Boolean;
    FDraging    : Boolean;
    FDragStart  : Integer;
    FDragStop   : Integer;
    FDragPane   : TSplitterPanel;
    FDragBar    : Integer;
    function  IsSplitter: Boolean;
    function  GetSplittPanes(Index: Integer): TSplitterPanel;
    function  GetSplittCount: Integer;
    function  GetSplittState: TSplittDirection;
    procedure SetBarWidth(Value: Byte);
    procedure WMSetCursor(var Message: TWMSetCursor); message WM_SetCursor;
  protected
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X,Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X,Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X,Y: Integer); override;
    procedure WorkspaceChange(Cause: TWorkspaceChange); override;
    procedure DragInit(X,Y: Integer);
    procedure DragMove(X,Y: Integer);
    procedure DragDone(X,Y: Integer);
    procedure DragPaint(Pos: Integer);
    procedure DragCheck(var Pos: Integer);
    procedure DragResize(Pos: Integer);
    function  SplitterHit(X,Y: Integer): Boolean;
    function  SplitterSize: Integer;
    procedure InsertPane(Pane: TSplitterPanel);
    procedure RemovePane(Pane: TSplitterPanel);
    function  LowerPane(Pos: Integer): TSplitterPanel;
    function  HigherPane(Pos: Integer): TSplitterPanel;
    procedure MakeSplitt(Pane1,Pane2: TSplitterPanel; Splitt: TSplittDirection);
    procedure EditSplitt(Pane: TSplitterPanel);
    procedure SortPanes;
    procedure AdjustPanes;
    property  SplittPanes[Index: Integer]: TSplitterPanel read GetSplittPanes;
    property  SplittCount: Integer read GetSplittCount;
    property  SplittState: TSplittDirection read GetSplittState {write FSplittState};
  public
    procedure RegistrySave(Key: HKey; Path: String);
    procedure RegistryLoad(Key: HKey; Path: String);
  published
    property  BarStyle: TSplittBarStyle read FBarStyle write FBarStyle stored IsSplitter;
    property  BarWidth: Byte read FBarWidth write SetBarWidth stored IsSplitter;
    property  CursorHorz: TCursor read FCursorHorz write FCursorHorz stored IsSplitter;
    property  CursorVert: TCursor read FCursorVert write FCursorVert stored IsSplitter;

  {pane functionality}
  private
    FPanePos    : Integer;
    FPaneSize   : Integer;
    FWantedSize : Integer;
    FMinimumSize: Integer;
    FAllowEdit  : Boolean;
    function  IsPane: Boolean;
    function  GetSplitter: TSplitterPanel;
    procedure SetSplitter(Value: TSplitterPanel);
    function  GetDirection: TSplittDirection;
    function  GetPaneShow: Boolean;
    function  GetPanePos: Integer;
    function  GetPaneSize: Integer;
    function  GetWantedSize: Integer;
    procedure SetMinimumSize(Value: Integer);
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
    procedure WMMove(var Message: TWMMove); message WM_Move;
    procedure WMSize(var Message: TWMSize); message WM_Size;
  protected
    procedure SetParent(AParent: TWinControl); override;
    procedure UpdatePane;
    property  Splitter: TSplitterPanel read GetSplitter write SetSplitter;
    property  Direction: TSplittDirection read GetDirection;
    property  PaneShow: Boolean read GetPaneShow;
    property  PanePos: Integer read GetPanePos write FPanePos;
    property  PaneSize: Integer read GetPaneSize write FPaneSize;
    property  WantedSize: Integer read GetWantedSize write FWantedSize;
  public
  published
    property  MinimumSize: Integer read FMinimumSize write SetMinimumSize stored IsPane;
end;

type
  TSplitterPanelEditor = class(TDefaultEditor)
    function  GetVerbCount: Integer; override;
    function  GetVerb(Index: Integer): string; override;
    procedure ExecuteVerb(Index: Integer); override;
  end;

procedure Register;

{==============================================================================}
implementation

{- TSplitterPanel -------------------------------------------------------------}

constructor TSplitterPanel.Create;
begin
  inherited Create(AOwner);
  Height := 145;
  Width := 145;
  BevelOuter.Style := bvNone;
  FCursorVert := crSizeWE;
  FCursorHorz := crSizeNS;
  FSplittState := spNonSplitt;
  FBarWidth := 3;
  FBarStyle := spContent;
  FSplittPanes := nil;
  FSortedPanes := true;
  FDraging := false;
  FPanePos := -1;
  FPaneSize := -1;
  FWantedSize := -1;
  FMinimumSize := 4;
  FAllowEdit := false;
end;

destructor TSplitterPanel.Destroy;
begin
  if Splitter <> nil then Splitter.RemovePane(Self);
  if FSplittPanes <> nil then FSplittPanes.Free;
  inherited;
end;

procedure TSplitterPanel.CMDesignHitTest;
begin
  with Message do
    if SplitterHit(Pos.X,Pos.Y) and (Keys and MK_LBUTTON <> 0) then Result := 1
    else begin
      if FDraging then Result := 1 // MK_LBUTTON not set in Keys on released
      else inherited;
    end;
end;

procedure TSplitterPanel.Loaded;
begin
  inherited;
  if not (csDesigning in ComponentState) then
    if IsSplitter then AdjustPanes;
end;

{ Splitter functionality }

function TSplitterPanel.IsSplitter;
begin
  Result := SplittState <> spNonSplitt;
end;

function TSplitterPanel.GetSplittPanes;
begin
  if not FSortedPanes then SortPanes;
  if (Index < 0) or (Index >= SplittCount) then Result := nil
  else Result := TSplitterPanel(FSplittPanes[Index]);
end;

function TSplitterPanel.GetSplittCount;
begin
  if FSplittPanes = nil then Result := 0
  else Result := FSplittPanes.Count;
end;

function TSplitterPanel.GetSplittState;
var
  Pane0: TSplitterPanel;
  Pane1: TSplitterPanel;
begin
  if (FSplittState = spUnknown) and (SplittCount > 1) then begin
    Pane0 := TSplitterPanel(FSplittPanes[0]);
    Pane1 := TSplitterPanel(FSplittPanes[1]);
    if Pane0.Left <> Pane1.Left then FSplittState := spVertical
    else if Pane0.Top <> Pane1.Top then FSplittState := spHorizontal;
  end;
  Result := FSplittState;
end;

procedure TSplitterPanel.SetBarWidth;
begin
  if Value <> FBarWidth then begin
    FBarWidth := value;
    if not (csLoading in ComponentState) then AdjustPanes;
  end;
end;

procedure TSplitterPanel.WMSetCursor;
var
  Pos: TPoint;
  Cur: TCursor;
begin
  with Message do
    if (CursorWnd = Handle) and (HitTest = HTClient) then begin
      Cur := Cursor;
      GetCursorPos(Pos);
      Pos := ScreenToClient(Pos);
      if SplitterHit(Pos.X,Pos.Y) then
        case SplittState of
          spVertical  : Cur := FCursorVert;
          spHorizontal: Cur := FCursorHorz;
        end;
      Windows.SetCursor(Screen.Cursors[Cur]);
      Result := 1;
    end
    else inherited;
end;

procedure TSplitterPanel.MouseDown;
begin
  inherited;
  if Button = mbLeft then DragInit(X,Y);
end;

procedure TSplitterPanel.MouseMove;
begin
  inherited;
  if FDraging then DragMove(X,Y);
end;

procedure TSplitterPanel.MouseUp;
begin
  inherited;
  if FDraging then DragDone(X,Y);
end;

procedure TSplitterPanel.WorkspaceChange;
begin
  inherited;
  if not (csLoading in ComponentState) then
    if IsSplitter then AdjustPanes;
end;

procedure TSplitterPanel.DragInit;
var
  DragRect: TRect;
begin
  if SplitterHit(X,Y) then begin
    FDraging := true;
    case SplittState of
      spVertical  : FDragStart := X-GetWorkspace.Left;
      spHorizontal: FDragStart := Y-GetWorkspace.Top;
    end;
    FDragStop := FDragStart;
    FDragPane := LowerPane(FDragStart);
    FDragBar := FDragPane.PanePos+FDragPane.PaneSize;
    if BarStyle <> spContent then DragPaint(FDragBar);
    DragRect := GetWorkspace;
    Inc(DragRect.Left);
    Inc(DragRect.Top);
    DragRect.TopLeft := ClientToScreen(DragRect.TopLeft);
    DragRect.BottomRight := ClientToScreen(DragRect.BottomRight);
    ClipCursor(@DragRect);
  end;
end;

procedure TSplitterPanel.DragMove;
var
  DragMove: Integer;
begin
  case SplittState of
    spVertical  : DragMove := X-GetWorkspace.Left;
    spHorizontal: DragMove := Y-GetWorkspace.Top;
  end;
  DragCheck(DragMove);
  if DragMove <> FDragStop then begin
    if BarStyle <> spContent then DragPaint(FDragBar+(FDragStop-FDragStart));
    if BarStyle <> spContent then DragPaint(FDragBar+(DragMove-FDragStart))
    else begin
      DragResize(DragMove);
      FDragStart := DragMove;
    end;
    FDragStop := DragMove;
  end;
end;

procedure TSplitterPanel.DragDone;
begin
  if BarStyle <> spContent then begin
    DragPaint(FDragBar+(FDragStop-FDragStart));
    DragResize(FDragStop);
  end;
  FDraging := false;
  ClipCursor(nil);
  if csDesigning in ComponentState then (Owner as TForm).Designer.Modified;
end;

procedure TSplitterPanel.DragPaint;
var
  BarRect     : TRect;
  BarDC       : HDC;
  CheckPattern: array[0..8] of Word;
  CheckBitmap : HBitmap;
  CheckBrush  : HBrush;
  OldBrush    : HBrush;
  Index       : Integer;
begin
  case SplittState of
    spVertical : begin
      BarRect.Left := GetWorkspace.Left+Pos;
      BarRect.Right := GetWorkspace.Left+Pos+BarWidth;
      BarRect.Top := GetWorkspace.Top;
      BarRect.Bottom := GetWorkspace.Bottom;
    end;
    spHorizontal : begin
      BarRect.Left := GetWorkspace.Left;
      BarRect.Right := GetWorkspace.Right;
      BarRect.Top := GetWorkspace.Top+Pos;
      BarRect.Bottom := GetWorkspace.Top+Pos+BarWidth;
    end;
  end;
  BarDC := GetDCEx(Handle,0,(DCX_Cache or DCX_ParentClip));
  case BarStyle of
    spCheckered : begin
      for Index := 0 to 8 do CheckPattern[Index] := Word($5555 shl (Index and 1));
      CheckBitmap := CreateBitmap(8,8,1,1,@CheckPattern);
      CheckBrush := CreatePatternBrush(CheckBitmap);
      DeleteObject(CheckBitmap);
      OldBrush := SelectObject(BarDC,CheckBrush);
      PatBlt(BarDC,BarRect.Left,BarRect.Top,BarRect.Right-BarRect.Left,
             BarRect.Bottom-BarRect.Top,PatInvert);
      SelectObject(BarDC,OldBrush);
      DeleteObject(CheckBrush);
    end;
    spSolid : InvertRect(BarDC,BarRect);
  end;
  ReleaseDC(Handle,BarDC);
end;

procedure TSplitterPanel.DragCheck;
var
  Index : Integer;
  Pane  : TSplitterPanel;
  MaxPos: Integer;
begin
  if FDragPane.PaneSize+(Pos-FDragStart) < FDragPane.MinimumSize then
    Pos := FDragStart+FDragPane.MinimumSize-FDragPane.PaneSize;
  MaxPos := SplitterSize;
  for Index := FSplittPanes.IndexOf(FDragPane)+1 to SplittCount-1 do begin
    Pane := SplittPanes[Index];
    MaxPos := MaxPos-BarWidth-Pane.MinimumSize;
  end;
  if FDragPane.PanePos+FDragPane.PaneSize+(Pos-FDragStart) > MaxPos then
    Pos := MaxPos-FDragPane.PanePos-FDragPane.PaneSize+FDragStart;
end;

procedure TSplitterPanel.DragResize;
begin
  FDragPane.WantedSize := FDragPane.PaneSize+(Pos-FDragStart);
  AdjustPanes;
end;

function TSplitterPanel.SplitterHit;
var
  Workspace: TRect;
begin
  Result := false;
  if IsSplitter then begin
    Workspace := GetWorkspace;
    if (X > Workspace.Left) and (X < Workspace.Right)
    and (Y > Workspace.Top) and (Y < Workspace.Bottom)
    then Result := true;
  end
end;

function TSplitterPanel.SplitterSize;
var
  Workspace: TRect;
begin
  Workspace := GetWorkspace;
  if SplittState = spHorizontal then Result := Workspace.Bottom-Workspace.Top
  else Result := Workspace.Right-Workspace.Left;
end;

procedure TSplitterPanel.InsertPane;
begin
  if FSplittPanes = nil then FSplittPanes := TList.Create;
  if FSplittPanes.IndexOf(Pane) < 0 then begin
    FSplittPanes.Add(Pane);
    FSortedPanes := false;
    if FSplittState = spNonSplitt then FSplittState := spUnknown;
  end;
end;

procedure TSplitterPanel.RemovePane;
begin
  if FSplittPanes = nil then Exit;
  if not FSplittPanes.IndexOf(Pane) < 0 then begin
    FSplittPanes.Remove(Pane);
    if SplittCount = 0 then begin
      FSplittState := spNonSplitt;
      FSplittPanes.Free;
      FSplittPanes := nil;
    end
    else if (SplittCount = 1) and (csDesigning in ComponentState) then
      SplittPanes[0].Free
    else if not (csDestroying in ComponentState) then
      AdjustPanes;
  end;
end;

function TSplitterPanel.LowerPane;
var
  Index : Integer;
  Pane  : TSplitterPanel;
  LowPos: Integer;
begin
  Result := nil;
  LowPos := 0;
  for Index := 0 to SplittCount-1 do begin
    Pane := SplittPanes[Index];
    if (Pane.PanePos < Pos) and (Pane.PanePos >= LowPos) then
      if Pane.PaneShow then begin
        Result := Pane;
        LowPos := Pane.PanePos;
      end;
  end;
end;

function TSplitterPanel.HigherPane;
var
  Index  : Integer;
  Pane   : TSplitterPanel;
  HighPos: Integer;
begin
  Result := nil;
  HighPos := SplitterSize;
  for Index := 0 to SplittCount-1 do begin
    Pane := SplittPanes[Index];
    if (Pane.PanePos > Pos) and (Pane.PanePos <= HighPos) then
      if Pane.PaneShow then begin
        Result := Pane;
        HighPos := Pane.PanePos;
      end;
  end;
end;

procedure TSplitterPanel.MakeSplitt;
var
  Length: Integer;
begin
  if SplittState = spNonSplitt then begin
    if Splitter <> nil then begin
      FBarStyle := Splitter.BarStyle;
      FBarWidth := Splitter.BarWidth;
      FCursorVert := Splitter.CursorVert;
      FCursorHorz := Splitter.CursorHorz;
    end;
    FSplittState := Splitt;
    Pane1.Border := Border;
    Pane1.BevelOuter := BevelOuter;
    Pane1.BevelInner := BevelInner;
    Pane1.PanePos := 0;
    Pane1.Splitter := Self;
    Length := SplitterSize;
  end
  else Length := Pane1.PaneSize;
  Pane1.WantedSize := Trunc((Length-BarWidth)/2);
  Pane2.Border := Pane1.Border;
  Pane2.BevelOuter := Pane1.BevelOuter;
  Pane2.BevelInner := Pane1.BevelInner;
  Pane2.PanePos := Pane1.PanePos+Pane1.WantedSize+BarWidth;
  Pane2.WantedSize := Length-BarWidth-Pane1.WantedSize;
  Pane2.Splitter := Self;
  AdjustPanes;
end;

procedure TSplitterPanel.EditSplitt;
var
  PrevPane      : TSplitterPanel;
  OldPos,OldSize: Integer;
  NewPos,NewSize: Integer;
begin
  PrevPane := LowerPane(Pane.PanePos);
  OldPos := Pane.PanePos;
  OldSize := Pane.PaneSize;
  Case SplittState of
    spVertical: begin
      NewPos := Pane.Left-GetWorkspace.Left;
      NewSize := Pane.Width;
    end;
    spHorizontal: begin
      NewPos := Pane.Top-GetWorkspace.Top;
      NewSize := Pane.Height;
    end;
    else begin
      NewPos := OldPos;
      NewSize := OldSize;
    end;
  end;

  if (NewPos <> OldPos) and (PrevPane = nil) then Pane.UpdatePane
  else if NewSize = OldSize then Pane.UpdatePane
  else begin
    if NewPos <> OldPos then PrevPane.WantedSize := PrevPane.PaneSize+NewPos-OldPos;
    Pane.WantedSize := NewSize;
    AdjustPanes;
  end;
end;

procedure TSplitterPanel.SortPanes;
var
  Index: Integer;
  Pane0: TSplitterPanel;
  Pane1: TSplitterPanel;
begin
  while not FSortedPanes do begin
    FSortedPanes := true;
    for Index := 1 to SplittCount-1 do begin
      Pane0 := TSplitterPanel(FSplittPanes[Index-1]);
      Pane1 := TSplitterPanel(FSplittPanes[Index]);
      if Pane1.PanePos < Pane0.PanePos then begin
        FSplittPanes.Exchange(Index,Index-1);
        FSortedPanes := false;
      end;
    end
  end;
end;

procedure TSplitterPanel.AdjustPanes;
var
  Index   : Integer;
  Pane    : TSplitterPanel;
  PrevPane: TSplitterPanel;
  NextPane: TSplitterPanel;
begin
  PrevPane := nil;
  for Index := 0 to SplittCount-1 do begin
    Pane := SplittPanes[Index];
    if Pane.PaneShow then begin
      if PrevPane = nil then Pane.PanePos := 0
      else Pane.PanePos := PrevPane.PanePos+PrevPane.PaneSize+BarWidth;
      Pane.PaneSize := Pane.WantedSize;
      PrevPane := Pane;
    end;
  end;

  NextPane := nil;
  for Index := SplittCount-1 downto 0 do begin
    Pane := SplittPanes[Index];
    if Pane.PaneShow then begin
      if NextPane = nil then begin
        if Pane.PanePos+Pane.MinimumSize > SplitterSize then begin
          Pane.PanePos := SplitterSize-Pane.MinimumSize;
          Pane.PaneSize := Pane.MinimumSize;
        end
        else Pane.PaneSize := SplitterSize-Pane.PanePos;
      end
      else begin
        if Pane.PanePos+Pane.MinimumSize > NextPane.PanePos-BarWidth then begin
          Pane.PanePos := NextPane.PanePos-BarWidth-Pane.MinimumSize;
          Pane.PaneSize := Pane.MinimumSize;
        end
        else Pane.PaneSize := NextPane.PanePos-BarWidth-Pane.PanePos;
      end;
      NextPane := Pane;
    end;
  end;

  PrevPane := nil;
  for Index := 0 to SplittCount-1 do begin
    Pane := SplittPanes[Index];
    if Pane.PaneShow then begin
      if PrevPane = nil then Pane.PanePos := 0
      else Pane.PanePos := PrevPane.PanePos+PrevPane.PaneSize+BarWidth;
      PrevPane := Pane;
    end;
  end;

  for Index := 0 to SplittCount-1 do begin
    Pane := SplittPanes[Index];
    if Pane.PaneShow then Pane.UpdatePane;
  end;

  Update; // process all painting before doing any thing else to avoid flicker
end;

type
  TSplitterPanelRegData = record
    PaneSize  : Integer;
    WantedSize: Integer;
    Visible   : Boolean;
  end;

procedure TSplitterPanel.RegistrySave;
var
  Index   : Integer;
  Pane    : TSplitterPanel;
  PaneData: TSplitterPanelRegData;
  Registry: TRegistry;
begin
  if not IsSplitter then Exit;
  Path := Path+'\'+Name;
  Registry := TRegistry.Create;
  Registry.RootKey := Key;
  try
    if Registry.OpenKey(Path,true) then
      for Index := 0 to SplittCount-1 do begin
        Pane := SplittPanes[Index];
        PaneData.PaneSize := Pane.PaneSize;
        PaneData.WantedSize := Pane.WantedSize;
        PaneData.Visible := Pane.Visible;
        Registry.WriteBinaryData(Pane.Name,PaneData,SizeOf(PaneData));
      end;
    Registry.CloseKey;
  finally
    Registry.Free;
  end;
  for Index := 0 to SplittCount-1 do SplittPanes[Index].RegistrySave(Key,Path);
end;

procedure TSplitterPanel.RegistryLoad;
var
  Index   : Integer;
  Pane    : TSplitterPanel;
  PaneData: TSplitterPanelRegData;
  Registry: TRegistry;
begin
  if not IsSplitter then Exit;
  Path := Path+'\'+Name;
  Registry := TRegistry.Create;
  Registry.RootKey := Key;
  try
    if Registry.OpenKey(Path,false) then
      for Index := 0 to SplittCount-1 do begin
        Pane := SplittPanes[Index];
        if Registry.ValueExists(Pane.Name) then
          if Registry.GetDataSize(Pane.Name) = SizeOf(PaneData) then begin
            Registry.ReadBinaryData(Pane.Name,PaneData,SizeOf(PaneData));
            Pane.PaneSize := PaneData.PaneSize;
            Pane.WantedSize := PaneData.WantedSize;
            Pane.Visible := PaneData.Visible;
          end;
      end;
    Registry.CloseKey;
  finally
    Registry.Free;
    AdjustPanes;
  end;
  for Index := 0 to SplittCount-1 do SplittPanes[Index].RegistryLoad(Key,Path);
end;

{ Pane functionality }

function TSplitterPanel.IsPane;
begin
  Result := Splitter <> nil;
end;

function TSplitterPanel.GetSplitter;
begin
  if Parent is TSplitterPanel then Result := Parent as TSplitterPanel
  else Result := nil;
end;

procedure TSplitterPanel.SetSplitter;
begin
  Parent := Value;
end;

function TSplitterPanel.GetDirection;
begin
  if Splitter = nil then Result := spNonSplitt
  else Result := Splitter.SplittState;
end;

function TSplitterPanel.GetPaneShow;
begin
  if csDesigning in ComponentState then Result := true
  else Result := Visible;
end;

function TSplitterPanel.GetPanePos;
begin
  if Direction in [spVertical,spHorizontal] then begin
    if FPanePos < 0 then
      case Direction of
        spVertical  : FPanePos := Left-Splitter.GetWorkspace.Left;
        spHorizontal: FPanePos := Top-Splitter.GetWorkspace.Top;
      end;
    Result := FPanePos;
  end
  else Result := -1;
end;

function TSplitterPanel.GetPaneSize;
begin
  if Direction in [spVertical,spHorizontal] then begin
    if FPaneSize < 0 then
      case Direction of
        spVertical  : FPaneSize := Width;
        spHorizontal: FPaneSize := Height;
      end;
    Result := FPaneSize;
  end
  else Result := -1;
end;

function TSplitterPanel.GetWantedSize;
begin
  if FWantedSize < 0 then FWantedSize := PaneSize;
  Result := FWantedSize;
end;

procedure TSplitterPanel.SetMinimumSize;
begin
  FMinimumSize := Value;
  if not (csLoading in ComponentState) then Splitter.AdjustPanes;
end;

procedure TSplitterPanel.CMVisibleChanged;
begin
  inherited;
  if not ((csDesigning in ComponentState) or (csLoading in ComponentState)) then
    if Splitter <> nil then Splitter.AdjustPanes;
end;

procedure TSplitterPanel.WMMove;
begin
  inherited;
  if IsPane and (csDesigning in ComponentState) then
    if not (csLoading in ComponentState) then begin
      if Align <> alNone then Align := alNone;
      if not FAllowEdit then Splitter.EditSplitt(Self);
    end;
end;

procedure TSplitterPanel.WMSize;
begin
  inherited;
  if IsPane and (csDesigning in ComponentState) then
    if not (csLoading in ComponentState) then begin
      if Align <> alNone then Align := alNone;
      if not FAllowEdit then Splitter.EditSplitt(Self);
    end;
end;

procedure TSplitterPanel.SetParent(AParent: TWinControl);
begin
  inherited;
  if Splitter <> nil then Splitter.InsertPane(Self);
end;

procedure TSplitterPanel.UpdatePane;
var
  Workspace: TRect;
  NewLeft  : Integer;
  NewTop   : Integer;
  NewWidth : Integer;
  NewHeight: Integer;
begin
  if Direction <> spNonSplitt then begin
    Workspace := Splitter.GetWorkspace;
    case Direction of
      spHorizontal: begin
        NewLeft := Workspace.Left;
        NewTop := Workspace.Top+PanePos;
        NewWidth := Workspace.Right-Workspace.Left;
        NewHeight := PaneSize;
      end;
      spVertical: begin
        NewLeft := Workspace.Left+PanePos;
        NewTop := Workspace.Top;
        NewWidth := PaneSize;
        NewHeight := Workspace.Bottom-Workspace.Top;
      end;
      else begin
        NewLeft := Left;
        NewTop := Top;
        NewWidth := Width;
        NewHeight := Height;
      end;
    end;
    FAllowEdit := true;
    SetBounds(NewLeft,NewTop,NewWidth,NewHeight);
    FAllowEdit := false
  end;
end;

{- TSplitterPanelEditor -------------------------------------------------------}

const
  specSplittVertical   = 0;
  specSplittHorizontal = 1;
  specDeletePane       = 2;

function TSplitterPanelEditor.GetVerbCount;
begin
  if not (Component as TSplitterPanel).IsSplitter then Result := 3
  else Result := 0;
end;

function TSplitterPanelEditor.GetVerb;
begin
  case Index of
    specSplittVertical  : Result := 'Splitt Vertical';
    specSplittHorizontal: Result := 'Splitt Horizontal';
    specDeletePane      : Result := 'Delete Pane';
  end;
end;

procedure TSplitterPanelEditor.ExecuteVerb;
var
  Direction    : TSplittDirection;
  Designer     : TFormDesigner;
  SplitterPanel: TSplitterPanel;
  Pane1        : TSplitterPanel;
  Pane2        : TSplitterPanel;

  function MakePane: TSplitterPanel;
  begin
    Result := TSplitterPanel.Create(Designer.Form);
    Result.Name := Designer.UniqueName(TSplitterPanel.ClassName);
  end;

begin
  Designer := Self.Designer;
  SplitterPanel := Component as TSplitterPanel;
  if Index in [specSplittHorizontal,specSplittVertical] then begin
    if SplitterPanel.IsSplitter then Exit;
    case Index of
      specSplittVertical  : Direction := spVertical;
      specSplittHorizontal: Direction := spHorizontal;
    end;
    if Direction <> SplitterPanel.Direction then Pane1 := MakePane
    else begin
      Pane1 := SplitterPanel;
      SplitterPanel := Pane1.Splitter;
    end;
    Pane2 := MakePane;
    SplitterPanel.MakeSplitt(Pane1,Pane2,Direction);
    Designer.SelectComponent(Pane2);
  end
  else if Index = specDeletePane then SplitterPanel.Free;
  Designer.Modified;
end;

{- Register -------------------------------------------------------------------}

procedure Register;
begin
  RegisterComponents(Register_SplitterPanel,[TSplitterPanel]);
  RegisterComponentEditor(TSplitterPanel,TSplitterPanelEditor);
end;

end.

